﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.CombinedIntegerVectorEncoding {
  public class UniformActionManipulator : CombinedIntegerVectorManipulator {

    public ILookupParameter<IItemSet<CombinedIntegerVector>> PossibleActionsParameter {
      get { return (ILookupParameter<IItemSet<CombinedIntegerVector>>)Parameters["PossibleActions"]; }
    }

    [StorableConstructor]
    protected UniformActionManipulator(bool deserializing) : base(deserializing) { }
    protected UniformActionManipulator(UniformActionManipulator original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new UniformActionManipulator(this, cloner);
    }
    public UniformActionManipulator()
      : base() {
      Parameters.Add(new LookupParameter<IItemSet<CombinedIntegerVector>>("PossibleActions"));
    }

    protected override CombinedIntegerVector Manipulate(IRandom random, CombinedIntegerVector child, CombinedIntegerVector input) {
      return Manipulate(random, child, PossibleActionsParameter.ActualValue);
    }

    public static CombinedIntegerVector Manipulate(IRandom random, CombinedIntegerVector child, IEnumerable<CombinedIntegerVector> possibleActions) {
      var otherActions = possibleActions.Except(((CombinedIntegerVector)child.Action).ToEnumerable());
      int actionIndex = random.Next(otherActions.Count());
      CombinedIntegerVector newAction = otherActions.ElementAt(actionIndex);
      return new CombinedIntegerVector((CombinedIntegerVector)child.Condition, newAction, child.Bounds);
    }
  }
}
